/*
 * Created on 30.09.2004 by laliluna
 *
 */
package de.laliluna.tutorial.library.ejb;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import de.laliluna.tutorial.library.interfaces.BookLocal;
import de.laliluna.tutorial.library.interfaces.BookLocalHome;
import de.laliluna.tutorial.library.interfaces.BookValue;

/**
 * @author laliluna
 *
 */

/**
 * XDoclet-based stateless session bean. The class must be declared
 * <code>public</code>, according to the EJB specification. <br>
 * 
 * To generate code: <br>
 * <ul>
 * <li>Add Standard EJB module to XDoclet project properties
 * <li>Customize XDoclet configuration
 * <li>Run XDoclet
 * </ul>
 * <br>
 * Please see the included XDoclet Overview and the XDoclet Reference in the
 * help system for details
 * 
 * @ejb.bean name = "LibraryManager" 
 * type = "Stateless" 
 * display-name = "LibraryManager" 
 * description = "LibraryManager EJB" 
 * view-type =  "both" 
 * jndi-name = "ejb/LibraryManagerHome"
 */
public abstract class LibraryManager implements SessionBean {

    /** The SessionContext */
    private SessionContext context;

    /**
     * returns a Value Object from the book when the book exists and is
     * available, else returns null
     * 
     * @ejb.interface-method view-type = "both"
     * @param primaryKey
     * @return @throws
     *                EJBException
     */
    public BookValue lendBook(Integer primaryKey) throws EJBException {

        BookLocal bookLocal = null;

        try {
            // HS 30.09.2004 get a context to look the home interface by JNDI

            InitialContext context = new InitialContext(System.getProperties());

            BookLocalHome bookLocalHome = (BookLocalHome) context
                    .lookup(BookLocalHome.JNDI_NAME);

            bookLocal = bookLocalHome.findByPrimaryKey(primaryKey);
        } catch (FinderException e) {
            // laliluna 30.09.2004 do nothing but log message
            e.printStackTrace();
        } catch (NamingException e) {
            // laliluna 30.09.2004 do nothing but log message
            e.printStackTrace();
        }

        // laliluna 30.09.2004 book is availabe than return valueObject else
        // return null
        if (bookLocal != null && bookLocal.getAvailable()) {
            bookLocal.setAvailable(false);
            return bookLocal.getBookValue();
        }

        else
            return null;
    }

    /**
     * returns book to library and sets available to true
     * @ejb.interface-method view-type = "both"
     * @param bookValue
     * @throws EJBException
     */
    public void returnBook(BookValue bookValue) throws EJBException {

        // laliluna 01.10.2004 check params
        if (bookValue != null) {
            BookLocal bookLocal = null;

            try {
                // HS 30.09.2004 get a context to look the home interface by
                // JNDI
                InitialContext context = new InitialContext(System
                        .getProperties());

                BookLocalHome bookLocalHome = (BookLocalHome) context
                        .lookup(BookLocalHome.JNDI_NAME);

                // laliluna 01.10.2004 find bookObject by primary key
                bookLocal = bookLocalHome.findByPrimaryKey(bookValue
                        .getPrimaryKey());
                // laliluna 01.10.2004 return book and set availability to true
                bookLocal.setAvailable(true);
            } catch (FinderException e) {
                // laliluna 30.09.2004 do nothing but log message
                e.printStackTrace();
            } catch (NamingException e) {
                // laliluna 30.09.2004 do nothing but log message
                e.printStackTrace();
            }
        }
    }

    /**
     * An <code>ejbCreate</code> method as required by the EJB specification.
     * <br>
     * 
     * The container calls the instances <code>ejbCreate</code> method whose
     * signature matches the signature of the <code>create</code> method
     * invoked by the client. The input parameters sent from the client are
     * passed to the <code>ejbCreate</code> method. Each session bean class
     * must have at least one <code>ejbCreate</code> method. The number and
     * signatures of a session beans <code>create</code> methods are specific
     * to each session bean class.
     * 
     * @throws CreateException
     *                      Thrown if the instance could not perform the function
     *                      requested by the container because of an system-level error.
     * 
     * @ejb.create-method
     */
    public void ejbCreate() throws CreateException {
    }

    /**
     * The <code>ejbActivate()</code> method as required by the EJB
     * specification. <br>
     * 
     * The activate method is called when the instance is activated from its
     * passive" state. The instance should acquire any resource that it has
     * released earlier in the <code>ejbPassivate()</code> method. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @throws EJBException
     *                      Thrown if the instance could not perform the function
     *                      requested by the container because of an system-level error.
     */
    public void ejbActivate() throws EJBException {
    }

    /**
     * The <code>ejbPassivate()</code> method as required by the EJB
     * specification. <br>
     * 
     * The activate method is called when the instance is activated from its
     * "passive" state. The instance should acquire any resource that it has
     * released earlier in the <code>ejbActivate()</code> method. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @throws EJBException
     *                      Thrown if the instance could not perform the function
     *                      requested by the container because of an system-level error.
     */
    public void ejbPassivate() throws EJBException {
    }

    /**
     * The <code>ejbRemove()</code> method as required by the EJB
     * specification. <br>
     * 
     * A container invokes this method before it ends the life of the session
     * object. This happens as a result of a client's invoking a remove
     * operation, or when a container decides to terminate the session object
     * after a timeout. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @throws EJBException
     *                      Thrown if the instance could not perform the function
     *                      requested by the container because of an system-level error.
     */
    public void ejbRemove() throws EJBException {
    }

    /**
     * Set the associated session context. The container calls this method after
     * the instance creation. <br>
     * 
     * The enterprise bean instance should store the reference to the context
     * object in an instance variable. <br>
     * 
     * This method is called with no transaction context.
     * 
     * @throws EJBException
     *                      Thrown if the instance could not perform the function
     *                      requested by the container because of an system-level error.
     */
    public void setSessionContext(SessionContext newContext)
            throws EJBException {
        context = newContext;
    }

}